### R code with model fitting and performance tests for Kramer et al. Ecosphere manuscript "Suitability of Great Lakes for invasive species based on global species distribution models and local aquatic habitat"
### This has 3 species, and there are 3 sections for each species
### Those sections are: 1. the model and performance, 2. Random cross-validation of the model and 3. Spatial cross-validation of the model


#set up workspace
rm(list=ls(all=TRUE))

##Packages
library(maptools)
library(dismo)
library(raster)
library(rgdal)
library(spatstat)
library(rworldmap)
library(rgeos)
library(geometry)
library(ncdf)
library(ecospat)

# Source auxiliary functions
setwd("~/Desktop/NOAA ENM models")
source('functions.R')               # functions for generic species distribution modeling - MAY NOT BE NEEDED DEPENDING ON IF THINNING ETC
source('range-bagging-functions.R') # fit and predict functions for range bagging; depends on package 'geometry'
source('convex-hull.R')             # fit and plot convex hull of environmental covariates in PCA space
source('plot-and-importance-rb.R')  # performs variable importance and marginal plots for the range bagging algorithm

# Species specific data preparation
#source('global-data.R')
#source('global-data-5min.R')
#Only need to do this once, the prepared data is then saved and read in later
#snakehead was saved in a different RData file than the other two
#load('global-data-5min-GMandKS-RB-2014.rda')
#load('global-data-5min-SNandHVandGC-RB-2014.rda')
##These have been combined into a single file with all the species data and training/test divisions used below
load('Kramer et al Ecosphere data.Rda')

#Additional functions
source('species-data.R')
source('CV-functions.R')

#Other data preparations are used for cross-validation in code below, these are in the .Rda loaded above
# snakehead <- species.data('Channa-argus.csv', environmental.data=bioclim.rescaled.5min, global.pca=bioclim.pca.5min,
#                               buffer=2000000) #This is the 2000 km buffer (in meters), using 80% of data for training
# golden.mussel <- species.data('Limnoperna-fortunei.csv', environmental.data=bioclim.rescaled.5min, global.pca=bioclim.pca.5min,
#                               buffer=2000000) #This is the 2000 km buffer (in meters), using 80% of data for training
# killer.shrimp <- species.data('Dikerogammarus-villosus.csv',environmental.data=bioclim.rescaled.5min, global.pca=bioclim.pca.5min,
#                               buffer=2000000) #This is the 2000 km buffer (in meters), using 80% of data for training




#Set up geographic data for the Great Lakes region and the world
GLBasin_extent<-extent(-93.5,-74,40.25,50.75) ##This is the full watershed extent for mapping
new.pts<-as.matrix(crop(bioclim.rescaled.5min,extent(GLBasin_extent)))
#new.pts<-as.matrix(crop(bioclim.rescaled.5min,extent(GLmap)))
world.pts<-as.matrix(bioclim.rescaled) #This will be used to project globally, 10 min data for file and memory size


##Make plots of the Bioclim variables, these have been standardized/rescaled previously
Bioclim.lab<-c("Mean temp","Mean diurnal range","Isothermality","Temp seasonality","Max temp","Min temp","Temp range","Temp of wettest",
               "Temp of driest","Temp of warmest","Temp of coldest","Annual precip","Precip of wettest period","Precip of driest period",
               "Precip seasonality","Precip wettest","Precip driest","Precip of warmest","Precip of coldest")
pdf("Great Lakes Bioclim figure.pdf",width=7.5,height=10)
par(mfrow=c(5,4),mar=c(1.4,2,2,3.75),mgp=c(2,0.5,0))
for (i in 1:19){
  plot(bioclim.rescaled.5min[[i]], main = Bioclim.lab[i],ext=extent(GLmap),legend.shrink=0.9) # Plot raster
  plot(GLmap,add=TRUE) # Add Great Lakes outlines
}
dev.off()


################Snakehead analysis#####################################################################################################################################

##Plot the occurrences of the Snakehead
# blank.map<-bioclim.rescaled[[1]]
# values(blank.map)<-ifelse(!is.na(values(blank.map)),1,NA)
# #x11(5,3)
# par(mar=c(3,3,2,1))
# spplot(blank.map,col.regions=c("wheat"),xlab="Longitude",ylab="Latitude",main="Snakehead occurrences",colorkey=F,
#        sp.layout=list(list("sp.points",snakehead[["sp.pts"]],col="red",cex=0.5,pch=3)
#                       #,list("sp.polygons",background,col="black",first=FALSE)
#                       ),
#        scales=list(draw=T))


##Define data included in the model
q<-19  ##This is the number of bioclim covariates to include in the following models. 19 is max with current data preparation.
#new.pts<-new.pts[,1:q] #Restricts to the variables used, here we use all 19
#world.pts<-world.pts[,1:q] #Restricts to the variables used, here we use all 19


#Fit model and apply to Great Lakes data set for mapping
set.seed(8272013)
p<-1 #Tuning parameter for proportion of points used in model fitting
model.rb.oc <- rb(x=snakehead[["train.presence"]][,1:q], v = 256, d=2, p=p)
saveRDS(model.rb.oc,"SN rb model-2016.rds")
# rb.oc.predictions <- rb.test(models = model.rb.oc, x.new = new.pts) #This makes predictions to the Great Lakes
# rb.oc.predictions.world <- rb.test(models = model.rb.oc, x.new = world.pts) #This predicts globally, is time consuming

#Predict on train data
rb.train.output.presence <- rb.test(models = model.rb.oc, x.new = snakehead[["train.presence"]][,1:q]) #Predict on training presence data
rb.train.output.back <- rb.test(models = model.rb.oc, x.new = snakehead[["train.background.balanced"]][,1:q])#Predict on training background data
#Predict on test data
rb.test.output.presence <- rb.test(models = model.rb.oc, x.new = snakehead[["test.presence"]][,1:q]) #Predict on test presence data
rb.test.output.back <- rb.test(models = model.rb.oc, x.new = snakehead[["test.background"]][,1:q]) #Predict on test background data


# #Generate figure of the predictions for the Great Lakes
# SNcolors1<-colorRampPalette(c("tan","yellow","green yellow","green","sea green"))
# SNcolors2<-colorRampPalette(c("white","light goldenrod 1","green yellow","lawn green","green","sea green","dark green"))
# output.rb.oc <- crop(raster(bioclim.rescaled.5min),extent(GLBasin_extent))  # Initialize raster
# values(output.rb.oc) <- rb.oc.predictions                         # Assign predictions to values of raster
# #x11(6,7)
# par(mfrow=c(2,1),mar=c(3,3,2,3))
# plot(output.rb.oc, main = "Great Lakes niche - Snakehead",col=c("white",SNcolors1(100)),breaks=seq(0,1,length.out=101),legend=F)
# plot(output.rb.oc, legend.only=T,col=c("white",SNcolors1(100)),breaks=seq(0,1,length.out=101),
#      axis.args=list(at=seq(0, 1, length.out=11),labels=seq(0, 1, length.out=11), cex.axis=0.6),
#      legend.args=list(text='Niche centrality', side=4, font=2, line=2.5, cex=0.8))                        # Plot raster
# plot(GLmap,add=TRUE)                                              # Add Great Lakes outlines
# 
#                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 #Output prediction raster to ESRI for Gust
# writeRaster(output.rb.oc,filename="~/Desktop/NOAA ENM models/Snakehead/Raster map - Snakehead/Snakehead map 2016.asc",format="ascii",overwrite=T)


##code for testing performance, requires background/absences
performance.rb.insample.sn <- evaluate(p = rb.train.output.presence,
                                 a = rb.train.output.back)
# plot(performance.rb.insample.sn, "ROC")
performance.rb.test.sn <- evaluate(p = rb.test.output.presence,
                                   a = rb.test.output.back)
rb.boyce.output<-rb.test(models=model.rb.oc,x.new=rbind(snakehead[["train.background"]][,1:q],snakehead[["test.background"]][,1:q]))
Boyce.sn<-ecospat.boyce(fit=rb.boyce.output,obs=rb.test.output.presence,PEplot=F)$Spearman.cor
saveRDS(list(performance.rb.test.sn,Boyce.sn),"SN evaluate test2016.rds")
thresh<-performance.rb.test.sn@t[which.max(performance.rb.test.sn@TPR + performance.rb.test.sn@TNR)]  
# pdf("Snakehead AUC on Test-July2016.pdf",width=4,height=4)
# plot(performance.rb.test.sn, "ROC")
# text(x=0.6,y=0.4,paste("MaxSens+Spec =",thresh))
# dev.off()



##Plot the global predictions, if desired
# output.rb.oc.world<-raster(bioclim.rescaled)
# values(output.rb.oc.world) <- rb.oc.predictions.world
# projection(output.rb.oc.world)<-projection(bioclim.rescaled)
# values(output.rb.oc.world)<-ifelse(is.na(values(bioclim.rescaled[[1]])),NA,values(output.rb.oc.world))
# writeRaster(output.rb.oc.world,filename="~/Desktop/NOAA ENM models/Snakehead/Raster map - Snakehead/Snakehead world map 2016.asc",format="ascii",overwrite=T)
# plot(output.rb.oc.world,main= "Global Niche - Snakehead",col=c("white",SNcolors2(100)),breaks=seq(0,1,length.out=101),legend=F)
# plot(output.rb.oc.world, legend.only=T,col=c("white",SNcolors2(100)),breaks=seq(0,1,length.out=101),
#      axis.args=list(at=seq(0, 1, length.out=11),labels=seq(0, 1, length.out=11), cex.axis=0.6),
#      legend.args=list(text='Niche centrality', side=4, font=2, line=2.5, cex=0.8))           


##Calculate variable importance and make marginal plots, only done for the main model, not for cross-validations
# importance.sn<-variable.imp.rb(model=model.rb.oc,
#                             model.auc.insample=performance.rb.insample.sn@auc, model.auc.test=performance.rb.test.sn@auc, 
#                             p=p, v=256,d=2,
#                             training.data=snakehead[["train.presence"]][,1:q],
#                             test.data=snakehead[["test.presence"]][,1:q],
#                             background=snakehead[["train.background.balanced"]][,1:q],
#                             test.back=snakehead[["test.background"]][,1:q],
#                             variables="ALL",n.permute=500,parallel=F)
# print(importance.sn); saveRDS(importance.sn,"SN importance-2016.rds")
# pdf("Snakehead importance2016.pdf",width=3,height=3)
# par(mar=c(3,6,1.5,0.1))
# plot(seq(1:dim(importance.sn)[1])~importance.sn[,"rel.imp.insample"],main="Relative importance of Bioclim variables/nSnakehead",
#      yaxt="n",xlab="Relative importance",ylab=NA)
# axis(2,at=seq(1:q),labels=rev(short.lab),las=1)
# mtext(side=2,line=5,"Variable name")
# dev.off()
# marginal<-marginal.plot.rb(model=model.rb.oc,training.data=snakehead[["train.presence"]][,1:q],
#                            coverage=100,p=1,variables="ALL",plot.file.name="SN-Marginal-plots-2016.pdf")
# saveRDS(marginal,"SN marginal plot2016.rds")


##Cross-validation for snakehead via random bins of data
type<-c(".pca","") ##To toggle between pca data and untransformed data
j<-2
folds<-10
cv.ids<-main.cv.folds(train.data=snakehead[["train.presence"]],background=snakehead[["train.background"]],folds=folds, r.seed=2162016) #Gives the data in each fold

##Make storage matrix
sn.cv.auc<-data.frame(matrix(NA,nrow=folds,ncol=2))
colnames(sn.cv.auc)<-c("AUC","Boyce")

for (k in 1:folds){
  #set.seed(4142016)  #This is the seed that makes range bagging match the range bagging ms
  ##Set up the cross-validation folds that will be used to tune the parameters
  ##This needs to be tuning cv-folds on the current overall training cv-fold
  p<- 1  #Fixed p, all points included
  model.rb.fix <- rb(x=snakehead[[paste("train.presence",type[j],sep="")]][cv.ids[[3]][[k]],1:q], v = 256, d=2, p=p)
  #Predict on training subset of validation data
  rb.cv.output.presence <- rb.test(models = model.rb.fix, x.new = snakehead[[paste("train.presence",type[j],sep="")]][cv.ids[[2]][[k]],1:q]) #Predict on test held out from the validation data
  rb.cv.output.back <- rb.test(models = model.rb.fix, x.new = snakehead[[paste("train.background",type[j],sep="")]][cv.ids[[4]][[k]],1:q])#Predict on background data with the same sample size as the validation test set
  ##code for testing performance, requires background/absences
  performance.rb.cv <- evaluate(p = rb.cv.output.presence,
                                a = rb.cv.output.back)
  ##Boyce index
  rb.cv.Boyce<-rb.test(models=model.rb.fix,x.new=rbind(snakehead[[paste("train.background",type[j],sep="")]][,1:q],snakehead[[paste("test.background",type[j],sep="")]][,1:q]))
  
  sn.cv.auc[k,1]<-performance.rb.cv@auc
  sn.cv.auc[k,2]<-ecospat.boyce(fit=rb.cv.Boyce,obs=rb.cv.output.presence,PEplot=F)$Spearman.cor
}
saveRDS(sn.cv.auc,"SN-ms-CV-2016.rds")

##Cross-validation for snakehead via spatial segregation of data
source("species-data-SS.R") #Note that these function replace the "species.data.R" functions, will be used for following species

validation.fraction<-0.2   #This is proportion of training data to hold out for validation in each cv-fold, 0.4 for golden mussel due to smaller sample size
if (validation.fraction==0.2) combns<-cbind(1,2,3,4,5)  
if (validation.fraction==0.4) combns<-cbind(c(1,2),c(2,3),c(3,4),c(4,5),c(5,1))#These are the possible combinations of longitudinal strips given a specific validation fraction, there are 5 strips and they are continguous in the case of two strips used as test
n.folds<-5  ##This is the number of combinations of longitudinal strips over which to test

##Make storage matrix
sn.ss.auc<-data.frame(matrix(NA,nrow=n.folds,ncol=2))
colnames(sn.ss.auc)<-c("AUC","Boyce")

snakehead.ss <- species.data('Channa-argus.csv',environmental.data=bioclim.rescaled.5min, global.pca=bioclim.pca.5min, 
                          buffer=2000000, train.fraction=0.8,validation.fraction=validation.fraction,check.map=F)
set.seed(3192015)
background.split<-c(rep(1,dim(snakehead.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(2,dim(snakehead.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(3,dim(snakehead.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(4,dim(snakehead.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(5,dim(snakehead.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds)) ##This is a vector to select subset of background points for each cv, only set up for 5 folds
for (k in 1:n.folds){
  test.strip<-combns[,k] #Defines the combination of strips to use as the test set
  ##The following code selects the training and test data for this fold of the data
  ##If statement is needed to deal with the data structure when more than 1 strip is used for testing (>20% of data), currently only alternative is 40%
  if(length(test.strip)==1){
    train.data<-snakehead.ss[[paste("train.presence",type[j],sep="")]][snakehead.ss[["valid.spatial.split"]][,2]!=test.strip,1:q] ##This collects the data in the 80% being used for training.
    test.data<-snakehead.ss[[paste("train.presence",type[j],sep="")]][snakehead.ss[["valid.spatial.split"]][,2]==test.strip,1:q]  ##This collects the data in the 20% being used for test
    train.background<-snakehead.ss[[paste("train.background",type[j],sep="")]][background.split!=test.strip,1:q] ##This uses 80% of the background data available for training (i.e. 6400 points), does this is a sequential manner, so each cv.fold leaves out a different chunk of points
    test.background<-snakehead.ss[[paste("train.background",type[j],sep="")]][background.split==test.strip,1:q][1:dim(test.data)[1],1:q] ##This takes the remaining background and uses the first X as a test, where X = number of test.data.pca locations.
  }
  
  ##Range-bagging
  p<- 1  #Alternate fixed p
  model.rb.fix <- rb(x=train.data, v = 256, d=2, p=p)
  #Predict on training subset of validation data
  rb.validation.output.presence <- rb.test(models = model.rb.fix, x.new = test.data) #Predict on test held out from the validation data
  rb.validation.output.back <- rb.test(models = model.rb.fix, x.new = test.background)#Predict on background data with the same sample size as the validation test set
  ##code for testing performance, requires background/absences
  performance.rb.spatial <- evaluate(p = rb.validation.output.presence,
                                     a = rb.validation.output.back)
  sn.ss.auc[k,1]<-performance.rb.spatial@auc
  ##Boyce index
  rb.ss.Boyce<-rb.test(models=model.rb.fix,x.new=rbind(train.background,test.background))
  sn.ss.auc[k,2]<-ecospat.boyce(fit=rb.ss.Boyce,obs=rb.validation.output.presence,PEplot=F)$Spearman.cor
}

saveRDS(sn.ss.auc,"SN-ms-Spatial-CV-2016.rds")

##############Golden mussel############################################################################################################################################
##Plot the occurrences of Golden mussel
# blank.map<-bioclim.rescaled[[1]]
# values(blank.map)<-ifelse(!is.na(values(blank.map)),1,NA)
# #x11(5,3)
# par(mar=c(3,3,2,1))
# spplot(blank.map,col.regions=c("wheat"),xlab="Longitude",ylab="Latitude",main="Golden mussel occurrences",colorkey=F,
#        sp.layout=list(list("sp.points",golden.mussel[["sp.pts"]],col="red",cex=0.5,pch=3)
#                       #,list("sp.polygons",background,col="black",first=FALSE)
#                       ),
#        scales=list(draw=T))


#Fit model and apply to Great Lakes data set for mapping
set.seed(952013)
p<-1 #Tuning parameter for proportion of points used in model fitting
model.rb.oc <- rb(x=golden.mussel[["train.presence"]][,1:q], v = 256, d=2, p=p)
saveRDS(model.rb.oc,"GM rb model-2016.rds")
# rb.oc.predictions <- rb.test(models = model.rb.oc, x.new = new.pts) #This makes predictions to the Great Lakes
# rb.oc.predictions.world <- rb.test(models = model.rb.oc, x.new = world.pts) #This predicts globally, is time consuming

#Predict on train data
rb.train.output.presence <- rb.test(models = model.rb.oc, x.new = golden.mussel[["train.presence"]][,1:q]) #Predict on training presence data
rb.train.output.back <- rb.test(models = model.rb.oc, x.new = golden.mussel[["train.background.balanced"]][,1:q])#Predict on training background data
#Predict on test data
rb.test.output.presence <- rb.test(models = model.rb.oc, x.new = golden.mussel[["test.presence"]][,1:q]) #Predict on test presence data
rb.test.output.back <- rb.test(models = model.rb.oc, x.new = golden.mussel[["test.background"]][,1:q]) #Predict on test background data


# #Generate figure of the predictions for the Great Lakes
# SNcolors1<-colorRampPalette(c("tan","yellow","green yellow","green","sea green"))
# SNcolors2<-colorRampPalette(c("white","light goldenrod 1","green yellow","lawn green","green","sea green","dark green"))
# output.rb.oc <- crop(raster(bioclim.rescaled.5min),extent(GLBasin_extent))  # Initialize raster
# values(output.rb.oc) <- rb.oc.predictions                         # Assign predictions to values of raster
# #x11(6,7)
# par(mfrow=c(2,1),mar=c(3,3,2,3))
# plot(output.rb.oc, main = "Great Lakes niche - Golden mussel",col=c("white",SNcolors1(100)),breaks=seq(0,1,length.out=101),legend=F)
# plot(output.rb.oc, legend.only=T,col=c("white",SNcolors1(100)),breaks=seq(0,1,length.out=101),
#      axis.args=list(at=seq(0, 1, length.out=11),labels=seq(0, 1, length.out=11), cex.axis=0.6),
#      legend.args=list(text='Niche centrality', side=4, font=2, line=2.5, cex=0.8))                        # Plot raster
# plot(GLmap,add=TRUE)                                              # Add Great Lakes outlines
# 
#                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 #Output prediction raster to ESRI for Gust
# writeRaster(output.rb.oc,filename="~/Desktop/NOAA ENM models/Golden mussel/Raster map - Limnoperna/Limnoperna map 2016.asc",format="ascii",overwrite=T)

##code for testing performance, requires background/absences
performance.rb.insample.gm <- evaluate(p = rb.train.output.presence,
                                       a = rb.train.output.back)
# plot(performance.rb.insample.gm, "ROC")
performance.rb.test.gm <- evaluate(p = rb.test.output.presence,
                                   a = rb.test.output.back)
rb.boyce.output<-rb.test(models=model.rb.oc,x.new=rbind(golden.mussel[["train.background"]][,1:q],golden.mussel[["test.background"]][,1:q]))
Boyce.gm<-ecospat.boyce(fit=rb.boyce.output,obs=rb.test.output.presence,PEplot=F)$Spearman.cor
saveRDS(list(performance.rb.test.gm,Boyce.gm),"GM evaluate test2016.rds")
thresh<-performance.rb.test.gm@t[which.max(performance.rb.test.gm@TPR + performance.rb.test.gm@TNR)]  
# pdf("Golden Mussel AUC on Test-July2016.pdf",width=4,height=4)
# plot(performance.rb.test.gm, "ROC")
# text(x=0.6,y=0.4,paste("MaxSens+Spec =",thresh))
# dev.off()



##Plot the global predictions, if desired
# output.rb.oc.world<-raster(bioclim.rescaled)
# values(output.rb.oc.world) <- rb.oc.predictions.world
# projection(output.rb.oc.world)<-projection(bioclim.rescaled)
# values(output.rb.oc.world)<-ifelse(is.na(values(bioclim.rescaled[[1]])),NA,values(output.rb.oc.world))
# writeRaster(output.rb.oc.world,filename="~/Desktop/NOAA ENM models/Golden mussel/Raster map - Limnoperna/Limnoperna world map 2016.asc",format="ascii",overwrite=T)
# plot(output.rb.oc.world,main= "Global Niche - Golden Mussel",col=c("white",SNcolors2(100)),breaks=seq(0,1,length.out=101),legend=F)
# plot(output.rb.oc.world, legend.only=T,col=c("white",SNcolors2(100)),breaks=seq(0,1,length.out=101),
#      axis.args=list(at=seq(0, 1, length.out=11),labels=seq(0, 1, length.out=11), cex.axis=0.6),
#      legend.args=list(text='Niche centrality', side=4, font=2, line=2.5, cex=0.8))           


##Calculate variable importance and make marginal plots, only done for the main model, not for cross-validations
# importance.gm<-variable.imp.rb(model=model.rb.oc,
#                                model.auc.insample=performance.rb.insample.gm@auc, model.auc.test=performance.rb.test.gm@auc, 
#                                p=p, v=256,d=2,
#                                training.data=golden.mussel[["train.presence"]][,1:q],
#                                test.data=golden.mussel[["test.presence"]][,1:q],
#                                background=golden.mussel[["train.background.balanced"]][,1:q],
#                                test.back=golden.mussel[["test.background"]][,1:q],
#                                variables="ALL",n.permute=500,parallel=F)
# print(importance.gm); saveRDS(importance.gm,"GM importance-2016.rds")
# pdf("Golden mussel importance2016.pdf",width=3,height=3)
# par(mar=c(3,6,1.5,0.1))
# plot(seq(1:dim(importance.gm)[1])~importance.gm[,"rel.imp.insample"],main="Relative importance of Bioclim variables/nGolden mussel",
#      yaxt="n",xlab="Relative importance",ylab=NA)
# axis(2,at=seq(1:q),labels=rev(short.lab),las=1)
# mtext(side=2,line=5,"Variable name")
# dev.off()
# marginal<-marginal.plot.rb(model=model.rb.oc,training.data=golden.mussel[["train.presence"]][,1:q],
#                            coverage=100,p=1,variables="ALL",plot.file.name="GM-Marginal-plots-2016.pdf")
# saveRDS(marginal,"GM marginal plot2016.rds")


##Cross-validation for golden mussel via random bins of data
type<-c(".pca","") ##To toggle between pca data and untransformed data
j<-2
folds<-10
cv.ids<-main.cv.folds(train.data=golden.mussel[["train.presence"]],background=golden.mussel[["train.background"]],folds=folds, r.seed=2162016) #Gives the data in each fold

##Make storage matrix
gm.cv.auc<-data.frame(matrix(NA,nrow=folds,ncol=2))
colnames(gm.cv.auc)<-c("AUC","Boyce")

for (k in 1:folds){
  #set.seed(4142016)  #This is the seed that makes range bagging match the range bagging ms
  ##Set up the cross-validation folds that will be used to tune the parameters
  ##This needs to be tuning cv-folds on the current overall training cv-fold
  p<- 1  #Fixed p, all points included
  model.rb.fix <- rb(x=golden.mussel[[paste("train.presence",type[j],sep="")]][cv.ids[[3]][[k]],1:q], v = 256, d=2, p=p)
  #Predict on training subset of validation data
  rb.cv.output.presence <- rb.test(models = model.rb.fix, x.new = golden.mussel[[paste("train.presence",type[j],sep="")]][cv.ids[[2]][[k]],1:q]) #Predict on test held out from the validation data
  rb.cv.output.back <- rb.test(models = model.rb.fix, x.new = golden.mussel[[paste("train.background",type[j],sep="")]][cv.ids[[4]][[k]],1:q])#Predict on background data with the same sample size as the validation test set
  ##code for testing performance, requires background/absences
  performance.rb.cv <- evaluate(p = rb.cv.output.presence,
                                a = rb.cv.output.back)
  ##Boyce index
  rb.cv.Boyce<-rb.test(models=model.rb.fix,x.new=rbind(golden.mussel[[paste("train.background",type[j],sep="")]][,1:q],golden.mussel[[paste("test.background",type[j],sep="")]][,1:q]))
  
  gm.cv.auc[k,1]<-performance.rb.cv@auc
  gm.cv.auc[k,2]<-ecospat.boyce(fit=rb.cv.Boyce,obs=rb.cv.output.presence,PEplot=F)$Spearman.cor
}
saveRDS(gm.cv.auc,"GM-ms-CV-2016.rds")

##Cross-validation for golden mussel via spatial segregation of data
source("species-data-SS.R") #Note that these function replace the "species.data.R" functions, will be used for following species

validation.fraction<-0.4   #This is proportion of training data to hold out for validation in each cv-fold, 0.4 for golden mussel due to smaller sample size
if (validation.fraction==0.2) combns<-cbind(1,2,3,4,5)  
if (validation.fraction==0.4) combns<-cbind(c(1,2),c(2,3),c(3,4),c(4,5),c(5,1))#These are the possible combinations of longitudinal strips given a specific validation fraction, there are 5 strips and they are continguous in the case of two strips used as test
n.folds<-5  ##This is the number of combinations of longitudinal strips over which to test

##Make storage matrix
gm.ss.auc<-data.frame(matrix(NA,nrow=n.folds,ncol=2))
colnames(gm.ss.auc)<-c("AUC","Boyce")

golden.mussel.ss <- species.data('Limnoperna-fortunei.csv',environmental.data=bioclim.rescaled.5min, global.pca=bioclim.pca.5min, 
                             buffer=2000000, train.fraction=0.8,validation.fraction=validation.fraction,check.map=F)
set.seed(3192015)
background.split<-c(rep(1,dim(golden.mussel.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(2,dim(golden.mussel.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(3,dim(golden.mussel.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(4,dim(golden.mussel.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(5,dim(golden.mussel.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds)) ##This is a vector to select subset of background points for each cv, only set up for 5 folds
for (k in 1:n.folds){
  test.strip<-combns[,k] #Defines the combination of strips to use as the test set
  ##The following code selects the training and test data for this fold of the data
  ##If statement is needed to deal with the data structure when more than 1 strip is used for testing (>20% of data), currently only alternative is 40%, used for golden mussel
  if(length(test.strip)==2){
    train.data<-golden.mussel.ss[[paste("train.presence",type[j],sep="")]][golden.mussel.ss[["valid.spatial.split"]][,2]!=test.strip[1] & golden.mussel.ss[["valid.spatial.split"]][,2]!=test.strip[2],1:q] ##This collects the data in the 60% being used for training.
    test.data<-golden.mussel.ss[[paste("train.presence",type[j],sep="")]][golden.mussel.ss[["valid.spatial.split"]][,2]==test.strip[1] | golden.mussel.ss[["valid.spatial.split"]][,2]==test.strip[2],1:q] ##This collects the data in the 40% being used for test
    train.background<-golden.mussel.ss[[paste("train.background",type[j],sep="")]][background.split!=test.strip[1] & background.split!=test.strip[2],1:q]  ##This uses 60% of the background data available for training (i.e. 4800 points), does this is a sequential manner, so each cv.fold leaves out a different chunk of points
    test.background<-golden.mussel.ss[[paste("train.background",type[j],sep="")]][background.split==test.strip[1] | background.split==test.strip[2],1:q] ##This takes the remaining background and uses the first X as a test, where X = number of test.data.pca locations.
  }
  ##Range-bagging
  p<- 1  #Alternate fixed p
  model.rb.fix <- rb(x=train.data, v = 256, d=2, p=p)
  #Predict on training subset of validation data
  rb.validation.output.presence <- rb.test(models = model.rb.fix, x.new = test.data) #Predict on test held out from the validation data
  rb.validation.output.back <- rb.test(models = model.rb.fix, x.new = test.background)#Predict on background data with the same sample size as the validation test set
  ##code for testing performance, requires background/absences
  performance.rb.spatial <- evaluate(p = rb.validation.output.presence,
                                     a = rb.validation.output.back)
  gm.ss.auc[k,1]<-performance.rb.spatial@auc
  ##Boyce index
  rb.ss.Boyce<-rb.test(models=model.rb.fix,x.new=rbind(train.background,test.background))
  gm.ss.auc[k,2]<-ecospat.boyce(fit=rb.ss.Boyce,obs=rb.validation.output.presence,PEplot=F)$Spearman.cor
}
saveRDS(gm.ss.auc,"GM-ms-Spatial-CV-2016.rds")



################Killer shrimp analysis#####################################################################################################################################

##Plot the occurrences of killer shrimp
# blank.map<-bioclim.rescaled[[1]]
# values(blank.map)<-ifelse(!is.na(values(blank.map)),1,NA)
#x11(5,3)
par(mar=c(3,3,2,1))
spplot(blank.map,col.regions=c("wheat"),xlab="Longitude",ylab="Latitude",main="Killer shrimp occurrences",colorkey=F,
       sp.layout=list(list("sp.points",killer.shrimp[["sp.pts"]],col="red",cex=0.5,pch=3)
                      #,list("sp.polygons",background,col="black",first=FALSE)
       ),
       scales=list(draw=T))


##Define data included in the model
q<-19  ##This is the number of bioclim covariates to include in the following models. 19 is max with current data preparation.
new.pts<-new.pts[,1:q] #Restricts to the variables used, here we use all 19
world.pts<-world.pts[,1:q] #Restricts to the variables used, here we use all 19


#Fit model and apply to Great Lakes data set for mapping
set.seed(10232014)
p<-1 #Tuning parameter for proportion of points used in model fitting
model.rb.oc <- rb(x=killer.shrimp[["train.presence"]][,1:q], v = 256, d=2, p=p)
saveRDS(model.rb.oc,"KS rb model-2016.rds")
# rb.oc.predictions <- rb.test(models = model.rb.oc, x.new = new.pts) #This makes predictions to the Great Lakes
# rb.oc.predictions.world <- rb.test(models = model.rb.oc, x.new = world.pts) #This predicts globally, is time consuming

#Predict on train data
rb.train.output.presence <- rb.test(models = model.rb.oc, x.new = killer.shrimp[["train.presence"]][,1:q]) #Predict on training presence data
rb.train.output.back <- rb.test(models = model.rb.oc, x.new = killer.shrimp[["train.background.balanced"]][,1:q])#Predict on training background data
#Predict on test data
rb.test.output.presence <- rb.test(models = model.rb.oc, x.new = killer.shrimp[["test.presence"]][,1:q]) #Predict on test presence data
rb.test.output.back <- rb.test(models = model.rb.oc, x.new = killer.shrimp[["test.background"]][,1:q]) #Predict on test background data


# #Generate figure of the predictions for the Great Lakes
# SNcolors1<-colorRampPalette(c("tan","yellow","green yellow","green","sea green"))
# SNcolors2<-colorRampPalette(c("white","light goldenrod 1","green yellow","lawn green","green","sea green","dark green"))
# output.rb.oc <- crop(raster(bioclim.rescaled.5min),extent(GLBasin_extent))  # Initialize raster
# values(output.rb.oc) <- rb.oc.predictions                         # Assign predictions to values of raster
# #x11(6,7)
# par(mfrow=c(2,1),mar=c(3,3,2,3))
# plot(output.rb.oc, main = "Great Lakes niche - Killer Shrimp",col=c("white",SNcolors1(100)),breaks=seq(0,1,length.out=101),legend=F)
# plot(output.rb.oc, legend.only=T,col=c("white",SNcolors1(100)),breaks=seq(0,1,length.out=101),
#      axis.args=list(at=seq(0, 1, length.out=11),labels=seq(0, 1, length.out=11), cex.axis=0.6),
#      legend.args=list(text='Niche centrality', side=4, font=2, line=2.5, cex=0.8))                        # Plot raster
# plot(GLmap,add=TRUE)                                              # Add Great Lakes outlines
# 
#                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 #Output prediction raster to ESRI for Gust
# writeRaster(output.rb.oc,filename="~/Desktop/NOAA ENM models/Killer shrimp/Raster map - Dikerogammarus/Dikerogammarus map 2016.asc",format="ascii",overwrite=T)



##code for testing performance, requires background/absences
performance.rb.insample.ks <- evaluate(p = rb.train.output.presence,
                                       a = rb.train.output.back)
# plot(performance.rb.insample.ks, "ROC")
performance.rb.test.ks <- evaluate(p = rb.test.output.presence,
                                   a = rb.test.output.back)
rb.boyce.output<-rb.test(models=model.rb.oc,x.new=rbind(killer.shrimp[["train.background"]][,1:q],killer.shrimp[["test.background"]][,1:q]))
Boyce.ks<-ecospat.boyce(fit=rb.boyce.output,obs=rb.test.output.presence,PEplot=F)$Spearman.cor
saveRDS(list(performance.rb.test.ks,Boyce.ks),"KS evaluate test2016.rds")
thresh<-performance.rb.test.ks@t[which.max(performance.rb.test.ks@TPR + performance.rb.test.ks@TNR)]  
# pdf("Killer shrimp AUC on Test-July2016.pdf",width=4,height=4)
# plot(performance.rb.test.ks, "ROC")
# text(x=0.6,y=0.4,paste("MaxSens+Spec =",thresh))
# dev.off()



##Plot the global predictions, if desired
# output.rb.oc.world<-raster(bioclim.rescaled)
# values(output.rb.oc.world) <- rb.oc.predictions.world
# projection(output.rb.oc.world)<-projection(bioclim.rescaled)
# values(output.rb.oc.world)<-ifelse(is.na(values(bioclim.rescaled[[1]])),NA,values(output.rb.oc.world))
# writeRaster(output.rb.oc.world,filename="~/Desktop/NOAA ENM models/Killer shrimp/Raster map - Dikerogammarus/Dikerogammarus world map 2016.asc",format="ascii",overwrite=T)
# plot(output.rb.oc.world,main= "Global Niche - Snakehead",col=c("white",SNcolors2(100)),breaks=seq(0,1,length.out=101),legend=F)
# plot(output.rb.oc.world, legend.only=T,col=c("white",SNcolors2(100)),breaks=seq(0,1,length.out=101),
#      axis.args=list(at=seq(0, 1, length.out=11),labels=seq(0, 1, length.out=11), cex.axis=0.6),
#      legend.args=list(text='Niche centrality', side=4, font=2, line=2.5, cex=0.8))           


##Calculate variable importance and make marginal plots, only done for the main model, not for cross-validations
# importance.ks<-variable.imp.rb(model=model.rb.oc,
#                                model.auc.insample=performance.rb.insample.ks@auc, model.auc.test=performance.rb.test.ks@auc, 
#                                p=p, v=256,d=2,
#                                training.data=killer.shrimp[["train.presence"]][,1:q],
#                                test.data=killer.shrimp[["test.presence"]][,1:q],
#                                background=killer.shrimp[["train.background.balanced"]][,1:q],
#                                test.back=killer.shrimp[["test.background"]][,1:q],
#                                variables="ALL",n.permute=500,parallel=F)
# print(importance.ks); saveRDS(importance.ks,"KS importance-2016.rds")
# pdf("Killer shrimp importance2016.pdf",width=3,height=3)
# par(mar=c(3,6,1.5,0.1))
# plot(seq(1:dim(importance.ks)[1])~importance.ks[,"rel.imp.insample"],main="Relative importance of Bioclim variables/nKiller shrimp",
#      yaxt="n",xlab="Relative importance",ylab=NA)
# axis(2,at=seq(1:q),labels=rev(short.lab),las=1)
# mtext(side=2,line=5,"Variable name")
# dev.off()
# marginal<-marginal.plot.rb(model=model.rb.oc,training.data=killer.shrimp[["train.presence"]][,1:q],
#                            coverage=100,p=1,variables="ALL",plot.file.name="KS-Marginal-plots-2016.pdf")
# saveRDS(marginal,"KS marginal plot2016.rds")


##Cross-validation for killer shrimp via random bins of data
type<-c(".pca","") ##To toggle between pca data and untransformed data
j<-2
folds<-10
cv.ids<-main.cv.folds(train.data=killer.shrimp[["train.presence"]],background=killer.shrimp[["train.background"]],folds=folds, r.seed=2162016) #Gives the data in each fold

##Make storage matrix
ks.cv.auc<-data.frame(matrix(NA,nrow=folds,ncol=2))
colnames(ks.cv.auc)<-c("AUC","Boyce")

for (k in 1:folds){
  #set.seed(4142016)  #This is the seed that makes range bagging match the range bagging ms
  ##Set up the cross-validation folds that will be used to tune the parameters
  ##This needs to be tuning cv-folds on the current overall training cv-fold
  p<- 1  #Fixed p, all points included
  model.rb.fix <- rb(x=killer.shrimp[[paste("train.presence",type[j],sep="")]][cv.ids[[3]][[k]],1:q], v = 256, d=2, p=p)
  #Predict on training subset of validation data
  rb.cv.output.presence <- rb.test(models = model.rb.fix, x.new = killer.shrimp[[paste("train.presence",type[j],sep="")]][cv.ids[[2]][[k]],1:q]) #Predict on test held out from the validation data
  rb.cv.output.back <- rb.test(models = model.rb.fix, x.new = killer.shrimp[[paste("train.background",type[j],sep="")]][cv.ids[[4]][[k]],1:q])#Predict on background data with the same sample size as the validation test set
  ##code for testing performance, requires background/absences
  performance.rb.cv <- evaluate(p = rb.cv.output.presence,
                                a = rb.cv.output.back)
  ##Boyce index
  rb.cv.Boyce<-rb.test(models=model.rb.fix,x.new=rbind(killer.shrimp[[paste("train.background",type[j],sep="")]][,1:q],killer.shrimp[[paste("test.background",type[j],sep="")]][,1:q]))
  
  ks.cv.auc[k,1]<-performance.rb.cv@auc
  ks.cv.auc[k,2]<-ecospat.boyce(fit=rb.cv.Boyce,obs=rb.cv.output.presence,PEplot=F)$Spearman.cor
}
saveRDS(ks.cv.auc,"KS-ms-CV-2016.rds")

##Cross-validation for killer shrimp via spatial segregation of data
source("species-data-SS.R") #Note that these function replace the "species.data.R" functions, will be used for following species

validation.fraction<-0.2   #This is proportion of training data to hold out for validation in each cv-fold, 0.4 for golden mussel due to smaller sample size
if (validation.fraction==0.2) combns<-cbind(1,2,3,4,5)  
if (validation.fraction==0.4) combns<-cbind(c(1,2),c(2,3),c(3,4),c(4,5),c(5,1))#These are the possible combinations of longitudinal strips given a specific validation fraction, there are 5 strips and they are continguous in the case of two strips used as test
n.folds<-5  ##This is the number of combinations of longitudinal strips over which to test

##Make storage matrix
ks.ss.auc<-data.frame(matrix(NA,nrow=n.folds,ncol=2))
colnames(ks.ss.auc)<-c("AUC","Boyce")

killer.shrimp.ss <- species.data('Dikerogammarus-villosus.csv',environmental.data=bioclim.rescaled.5min, global.pca=bioclim.pca.5min, 
                             buffer=2000000, train.fraction=0.8,validation.fraction=validation.fraction,check.map=F)
set.seed(3192015)
background.split<-c(rep(1,dim(killer.shrimp.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(2,dim(killer.shrimp.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(3,dim(killer.shrimp.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(4,dim(killer.shrimp.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds),
                    rep(5,dim(killer.shrimp.ss[[paste("train.background",type[j],sep="")]])[1]/n.folds)) ##This is a vector to select subset of background points for each cv, only set up for 5 folds
for (k in 1:n.folds){
  test.strip<-combns[,k] #Defines the combination of strips to use as the test set
  ##The following code selects the training and test data for this fold of the data
  ##If statement is needed to deal with the data structure when more than 1 strip is used for testing (>20% of data), currently only alternative is 40%
  if(length(test.strip)==1){
    train.data<-killer.shrimp.ss[[paste("train.presence",type[j],sep="")]][killer.shrimp.ss[["valid.spatial.split"]][,2]!=test.strip,1:q] ##This collects the data in the 80% being used for training.
    test.data<-killer.shrimp.ss[[paste("train.presence",type[j],sep="")]][killer.shrimp.ss[["valid.spatial.split"]][,2]==test.strip,1:q]  ##This collects the data in the 20% being used for test
    train.background<-killer.shrimp.ss[[paste("train.background",type[j],sep="")]][background.split!=test.strip,1:q] ##This uses 80% of the background data available for training (i.e. 6400 points), does this is a sequential manner, so each cv.fold leaves out a different chunk of points
    test.background<-killer.shrimp.ss[[paste("train.background",type[j],sep="")]][background.split==test.strip,1:q][1:dim(test.data)[1],1:q] ##This takes the remaining background and uses the first X as a test, where X = number of test.data.pca locations.
  }
  
  ##Range-bagging
  p<- 1  #Alternate fixed p
  model.rb.fix <- rb(x=train.data, v = 256, d=2, p=p)
  #Predict on training subset of validation data
  rb.validation.output.presence <- rb.test(models = model.rb.fix, x.new = test.data) #Predict on test held out from the validation data
  rb.validation.output.back <- rb.test(models = model.rb.fix, x.new = test.background)#Predict on background data with the same sample size as the validation test set
  ##code for testing performance, requires background/absences
  performance.rb.spatial <- evaluate(p = rb.validation.output.presence,
                                     a = rb.validation.output.back)
  ks.ss.auc[k,1]<-performance.rb.spatial@auc
  ##Boyce index
  rb.ss.Boyce<-rb.test(models=model.rb.fix,x.new=rbind(train.background,test.background))
  ks.ss.auc[k,2]<-ecospat.boyce(fit=rb.ss.Boyce,obs=rb.validation.output.presence,PEplot=F)$Spearman.cor
}
saveRDS(ks.ss.auc,"KS-ms-Spatial-CV-2016.rds")
