#Script with support functions of Anopheles arabiensis modeling project
#Created: May 5, 2011 by John M. Drake (jdrake@uga.edu)
#Added to: June 6, 2014 by Andrew Kramer (kramera3@uga.edu)


#function to get species coordinates
get.sp.coords <- function(sp, plot=FALSE, bbox=NULL){
  require(sp)
  data0<-read.csv('AnGamb_spp.csv')
  data<-data.frame(longitude=data0$LONG[data0$SPECIES==sp], latitude=data0$LAT[data0$SPECIES==sp])
  if(plot==TRUE) plot(data)
  return(SpatialPoints(data, proj4string=CRS('+proj=longlat'), bbox=bbox))
}


#function to rescale a raster by subtracting mean and dividing by standard deviation
rescale.raster <- function(data0, data1=NULL){
  # Rescale a raster by subtracting mean and dividing by standard deviation
  # 
  # Args:
  #   data0: input raster to learn scaling
  #   data1 (optional): second raster to rescale according to the distribution of data0
  #
  # Returns:
  #   A rescaled raster the size of the input
  if(missing(data1)){
    raster <- (data0-cellStats(data0,'mean'))/cellStats(data0,'sd')
  }else{
    raster <- (data1-cellStats(data0,'mean'))/cellStats(data0,'sd')
  }
  return(raster)  
}

#function to rescale a raster by empirical cumulative distribution function
raster.ecdf <- function(data0, data1=NULL){
  # Compute empirical cumulative distribution function on values in a raster
  # Applies ecdf to raster (optionally a second raster) to return a scaled version
  #
  # Args:
  #   data0: input raster to learn scaling
  #   data1 (optional): second raster to rescale according to the ecdf of data0
  #
  # Returns:
  #   A rescaled raster the size of the input
  vals0 <- getValues(data0)
  fun <- ecdf(vals0)
  if(missing(data1)){
    raster <- raster(data0)
    vals <- vals0
  }else{
    raster <- raster(data1)
    vals <- getValues(data1)
  }
  values(raster) <- fun(vals)
  return(raster)
}

#function to take a list of coordinates and buffer radius and return a polygon of buffered area, plot optiona
buffer.to.polygon <- function(coordinates, radius, plot=FALSE){
  require(spatstat)
  c <- coordinates  #local variable for coordinates
  rad <- radius     #local variable for radius
  #now create disc polygons
  polys<-list() 
  for(i in 1:nrow(c)) {
    discbuff<-disc(radius=rad, centre=c(c$longitude[i], c$latitude[i])) 
    discpoly<-Polygon(rbind(cbind(discbuff$bdry[[1]]$x, y=discbuff$bdry[[1]]$y), c(discbuff$bdry[[1]]$x[1], y=discbuff$bdry[[1]]$y[1])))
    polys<-c(polys, discpoly)
  }
  #now list discs
  spolys<-list() 
  for(i in 1:length(polys)) { 
    spolybuff<-Polygons(list(polys[[i]]), ID=row.names(c)[i]) 
    spolys<-c(spolys, spolybuff) 
  } 
  #conver <- raster(data.not.arabiensis)            # create a RasterLayer with the extent of data.arabiensis
  polybuff<-SpatialPolygons(spolys) 
  #plot
  if(plot==TRUE){
    plot(polybuff) 
    points(c, col="red") 
  }
  return(polybuff)
}



#function to thin points from an underdispersed (cluterest) dataset to a given resolution, plot optional
thin.points <- function(points, resolution, plot=TRUE) {
  r <- raster(points)             # create a RasterLayer with the extent of points
  res(r) <- resolution            # set the resolution of the cells
  cell <- cellFromXY(r, points)   # get the cell number for each point
  dup <- duplicated(cell)         # find duplicates
  data.thin <- points[!dup, ]     # select points from unique cells
  if(plot==TRUE){
    p <- rasterToPolygons(r)      # display the results
    plot(p, border='gray')
    points(points)
    points(data.thin, cex=0.5, col='red', pch='x')
  }
  return(data.thin)
}

cv.folds <- function(data, folds=10, r.seed=4172013, random.points=F){
  #This function divides the training portion of dataset="data" into k="folds" training and test for cross-validation
  #Set up objects to define the cv.folds
  cv.train.ids<-list()
  cv.test.ids<-list()
  cv.background.train.ids<-list()
  cv.background.test.ids<-list()
  if (random.points==T) {train.data<-data[["train.validation.random"]]
  }else
  {train.data<-data[["train.validation"]]}
  #Define string that changes between pca and rescaled data
  #str<-ifelse(type=="pca",".pca","")
  ##First reorder the row ids to remove structure
  k <- folds #number of folds
  set.seed(r.seed) #Allows for replication of cv.folds  
  p.ids<-sample(seq(1,dim(train.data)[1]))  # reorder records randomly
  n.test<-floor((1/k)*dim(train.data)[1]) ##Find the size of the test bins (10%)
  p.bin<-seq(1:n.test) ##Initialize the test bin
  back.ids<-sample(seq(1,dim(data[["train.validation.background"]])[1]))
  n.back.test<-floor((1/k)*dim(data[["train.validation.background"]])[1]) ##Find the size of the test bins (10%)
  back.bin<-seq(1:n.back.test) ##Initialize the background test bin
  for (i in 1:k){
    cv.test.ids[[i]]<-p.ids[p.bin]
    cv.train.ids[[i]]<-seq(1,dim(train.data)[1])[-cv.test.ids[[i]]]
    cv.background.test.ids[[i]]<-back.ids[back.bin]
    cv.background.train.ids[[i]]<-seq(1,dim(data[["train.validation.background"]])[1])[-cv.background.test.ids[[i]]]
    p.bin<-p.bin+n.test; back.bin<-back.bin+n.back.test
  }
  return(list(folds, cv.test.ids, cv.train.ids, cv.background.test.ids, cv.background.train.ids))
}

objective.fit <- function(param, data, cv.ids, method="rb",type=".pca", q=9, random.points=F){
  #This function fits the SDM model to pre-defined cross-validation sets of the data. Used with "optimize" to tune fitting parameters
  #Currently works for range bagging and lobag-oc
  train.data.name<-ifelse(random.points==T,"train.validation.random","train.validation")
  str<-ifelse(type==".pca",".pca","") #Differentiates analysis on pca or rescaled variables
  k <- cv.ids[[1]] #number of folds
  cv.test.ids<-cv.ids[[2]]; cv.train.ids<-cv.ids[[3]]; cv.background.test.ids<-cv.ids[[4]]; cv.background.train.ids<-cv.ids[[5]]
  cv.performance<-rep(NA,k)
  if (method=="rb"){
    for (j in 1:k){
      model.rb.oc <- rb(x=data[[paste(train.data.name,str,sep="")]][cv.train.ids[[j]],1:q], v = 256, d=2, p=param) #Fit model to cv fold j
      #Predict on cv test data
      rb.cv.output.presence <- rb.test(models = model.rb.oc, x.new = data[[paste(train.data.name,str,sep="")]][cv.test.ids[[j]],1:q]) #Predict on training presence data
      rb.cv.output.back <- rb.test(models = model.rb.oc, x.new = data[[paste("train.validation.background",str,sep="")]][cv.background.test.ids[[j]],1:q])#Predict on training background data
      cv.performance[j]<-evaluate(p=rb.cv.output.presence,a=rb.cv.output.back)@auc    
    }
  }
  else{
    if (method=="lobag-oc"){
      for (j in 1:k){
        #For lobag-oc the sigma parameter is assumed to be passed on log scale
        model.lobag.oc <- lobag.oc(p=data[[paste(train.data.name,str,sep="")]][cv.train.ids[[j]],1:q], sigma=10^param, n.votes=128)
        #Predict on cv test data
        lobag.cv.output.presence <- predictSvm(model = model.lobag.oc, p = data[[paste(train.data.name,str,sep="")]][cv.test.ids[[j]],1:q])
        #Predict on training presence data
        lobag.cv.output.back <- predictSvm(model = model.lobag.oc, p = data[[paste("train.validation.background",str,sep="")]][cv.background.test.ids[[j]],1:q]) #Predict on training background data
        cv.performance[j]<-performance.lobag.oc <- evaluate(p = as.numeric(lobag.cv.output.presence$p.out),
                                                            a = as.numeric(lobag.cv.output.back$p.out))@auc   
      }
    }
    else {
      print("Invalid method")
    }
  }
  return(-mean(cv.performance))
}
