###This script is the functions for making marginal plots and testing variable importance for range bagging

marginal.plot.rb<-function(model,training.data,coverage=100,p=1,variables="ALL",plot.file.name="Marginal-plots.pdf"){
  ## Function to plot the marginal effects of predictors for range bagging models
  ## model = the range-bagging model fit to all the predictors
  ## training.data = the data used to fit the model
  ## coverage = the number of divisions of the range of a predictor at which to assess predictions
  ## p = the parameter "p" in range-bagging algorithm, i.e. the proportion of the observations to use
  ## variables = a vector of the predictors to produce plots for, pass as a vector of column numbers or "ALL"
  ## plot.file.name = the file to save the plots in
  
  ##Need to load the climate data in order to back-transform the values
  ClimateCovar<-brick("World5.grd")
  ClimateCovar.transformed <- brick("World5_transformed.grd")
  if (variables=="ALL") variables<-seq(1,dim(training.data)[2]) #Which variables are assessed is "ALL" is chosen, otherwise
  outputs<-list()
  for (i in variables){ #This loops through each predictor
    #step<-(max(training.data[,i])-min(training.data[,i]))/coverage
    xi<-seq(min(training.data[,i]),max(training.data[,i]),length.out=coverage) #A vector of increments in the current predictor across the observed range
    marginal<-rep(NA,coverage)
    for (j in 1:coverage){ ##Loops over the increments of the predictor
      temp.data<-training.data  
      temp.data[,i]<-xi[j]  ##The relevant column is changed to a single value from the predictor vector
      predicts<-rb.test(models=model,x.new=temp.data)  ##Uses the model to predict the niche centrality at these values
      marginal[j]<-mean(predicts)  ##Calculates the mean niche centrality for these values
    }
    ## The next line back-transforms the predictor to the original scale
    xi.back<-original.scale(x=xi,variable=i,coverage=coverage,ClimateCovar=ClimateCovar,ClimateCovar.transformed=ClimateCovar.transformed) ##This calls the function to back-transform the profile variable to original scale (see "functions.R"::'rescale-raster')
      ##Stores the values for further use
    outputs[[i]]<-as.data.frame(cbind(xi,xi.back,marginal))
  }
  pdf(plot.file.name,width=7.5,height=10) #Open pdf device to store plots
  par(mfrow=c(5,2),mar=c(3.5,3.5,0.1,0.1),mgp=c(2,0.5,0)) #Saves 10 plots per page
  for (i in variables){
  ## Draws the marginal plot
    scale.margin<-(outputs[[i]]$marginal-min(outputs[[i]]$marginal))/(max(outputs[[i]]$marginal)-min(outputs[[i]]$marginal))
    plot(scale.margin~outputs[[i]]$xi.back,type="l",ylim=c(0,1),ylab="Predicted niche centrality",xlab=names(training.data[i]))  ##Creates the marginal plot
  }
  dev.off()
  return(outputs)
}

original.scale<-function(x,variable,coverage=100,ClimateCovar,ClimateCovar.transformed){
  ## Function to backtransform the Worldclim Bioclim predictors to original scale
  ## x = the vector of transformed values to back-transform
  ## variable = the identity of the predictor as its column number (Bioclim ##)
  ## coverage = the number of increments in the vector x
  ## ClimateCovar = the untransformed, unscaled variables
  ## ClimateCovar.transformed = the log and ecdf tranformed but not rescaled version of ClimateCovar
  if (variable %in% c(1:11,15,20:43)) {  ##These are the Bioclim variables that were not log-transformed 
    mean<-cellStats(ClimateCovar.transformed[[variable]],"mean")
    sd<-cellStats(ClimateCovar.transformed[[variable]],"sd")
    x.back<-(x*sd)+mean  ##This backtransforms the sequence of profile points to the original scale (see 'rescale-raster' in "functions.R")
  }
  else {
    if (variable %in% c(18:19,44:55)){  ##These are the Bioclim variables that were log-transformed first
      mean<-cellStats(ClimateCovar.transformed[[variable]],"mean")
      sd<-cellStats(ClimateCovar.transformed[[variable]],"sd")
      x.back<-exp((x*sd)+mean)-1  ##This backtransforms the sequence of profile points to the original scale (see 'rescale-raster' in "functions.R")
    } 
    else { ##These are the variables that were rescaled with ecdf
      data<-ClimateCovar[[variable]]
      vals0<-getValues(data)
      fun<-ecdf(vals0)
      x.back<-quantile(fun,type=1,probs=seq(0,1,1/(coverage-1)))
    }
  }
  return(x.back)    
}

##This function is designed to use permutation to assess the importance of each variable to the model fit
variable.imp.rb<-function(model,model.auc.insample,model.auc.test,p=1,v=256,d=2,training.data,background,test.data,test.back,
                          variables="ALL",n.permute=10000,parallel=T){
  ##A function designed to estimate variable importance for the range-bagging algorithm using permutation
  ## model = the range-bagging model fit to all the predictors
  ## training.data = the data used to fit the model
  ## model.auc = the auc for the model fit to all predictors
  ## background = the background points used to estimate the model.auc
  ## n.permute = the number of permutations to test, changes computation time
  ## p = the parameter "p" in range-bagging algorithm, i.e. the proportion of the observations to use
  ## variables = a vector of the predictors to produce plots for, pass as a vector of column numbers or "ALL"
  ## parallel = Not yet implemented option to calculate on multiple processors
  
  if (variables=="ALL") variables<-seq(1,dim(training.data)[2])  #Default is calculation for each variable, otherwise column numbers are passed
  perm.auc.insample<-rep(NA,length(variables))  ## stores the mean auc from all permutations of a variable
  sd.perm.auc.insample<-rep(NA,length(variables))  ##stores the sd in auc from all permutations of a variable
  delta.auc.insample<-rep(NA,length(variables))  ##stores the difference between the model.auc and the perm.auc
  perm.auc.test<-rep(NA,length(variables))  ## stores the mean auc from all permutations of a variable
  sd.perm.auc.test<-rep(NA,length(variables))  ##stores the sd in auc from all permutations of a variable
  delta.auc.test<-rep(NA,length(variables))  ##stores the difference between the model.auc and the perm.auc
  for (i in variables){
    permuted.insample<-rep(NA,n.permute)
    permuted.test<-rep(NA,n.permute)
    for (j in 1:n.permute){
      temp.data<-training.data
      temp.data[,i]<-temp.data[sample(1:dim(temp.data)[1]),i] ##This permutes (reorders) the values of a single variable, leaving the remainder unchanged
      model.rb.perm <- rb(x=temp.data, v = 256, d=2, p=p) ##Fit new model on the permuted data
      rb.output.perm <- rb.test(models=model.rb.perm,x.new=training.data)  ##Test new model on the training data
      rb.output.back.perm <- rb.test(models=model.rb.perm,x.new=background) ##This is the predictions for the background points
      performance.insample.rb <- evaluate(p = rb.output.perm,
                                 a = rb.output.back.perm) ##Estimates performance via auc for the permuted dataset
      permuted.insample[j]<-performance.insample.rb@auc  ## Stores AUC for each permutation
      rb.output.perm.test <- rb.test(models=model.rb.perm,x.new=test.data) ##Evaluate new model on test data
      rb.output.back.test.perm <- rb.test(models=model.rb.perm,x.new=test.back)
      performance.test.rb <- evaluate(p=rb.output.perm.test,
                                      a=rb.output.back.test.perm) ##Estimates performance via auc on the test data
      permuted.test[j]<-performance.test.rb@auc
    }
    perm.auc.insample[i]<-mean(permuted.insample)  ##Mean of the AUCs of permutations
    sd.perm.auc.insample[i]<-sd(permuted.insample) ##SD of the AUCs of permutations
    delta.auc.insample[i]<-model.auc.insample-perm.auc.insample[i] ##model.auc - perm.auc, how much performance declined = importance
    perm.auc.test[i]<-mean(permuted.test) ##Mean of the AUCs of permuted models on the test data
    sd.perm.auc.test[i]<-mean(permuted.test) ##SD of the AUCs of permuted models on the test data
    delta.auc.test[i]<-model.auc.test-perm.auc.test[i]
  }
  rel.imp.insample<-delta.auc.insample/max(delta.auc.insample) ##Scales the importance by the most the variable with highest influence
  rel.imp.test<-delta.auc.test/max(delta.auc.test)
  return(cbind(perm.auc.insample,delta.auc.insample,rel.imp.insample,perm.auc.test,delta.auc.test,rel.imp.test))
}