# Support functions for Great Lakes predictive modeling
# Created April 4, 2013 by JMD and AMK

species.data <- function(filename, environmental.data, global.pca, cutoff=Inf, thin=-999, buffer=500000, backgr.n=10000, 
                         check.map=TRUE, train.fraction=0.8, validation.fraction=0.2,seed=6212013){
  # Objective: Read in species point occurrence data and return list of objects for species distribution modeling
  #
  # Arguments:
  #  filename: name of file contain raw species data
  #  environmental.data: raster brick containing environmental data for species distribution modeling
  #  global.pca: result of principal component analysis used to transform environmental data to PCA space
  #  cutoff: records with half maximum spatial error > cutoff excluded from analysis
  #  thin: points removed based on resolution
  #  buffer: radius of buffer for selection of background points (in meters)
  #  bkgrd.n: number of background points to sample, NOTE this was 1000 for much experimentation, has been updated to 10,000
  #  check.map: should a map be made to inspect buffer regions
  #  withhold.fraction: approximate fraction of data points to be used for training
    
  # Read in occurrences worldwide

  sp<-read.csv(filename,header=T)
  if (filename=="Hydrilla-verticillata.csv"){
    colnames(sp)<-c('Internal.ID','Longitude','Latitude','Error','Half.Error')
  } else {
    if (filename=="Grass-carp.csv"){
      colnames(sp)<-c('Internal.ID','Longitude','Latitude','Error','Half.Error','Date.observed','Status')
    }else{
      colnames(sp)<-c('Internal.ID','Longitude','Latitude','Error','Half.Error','Native')
    }
  }
  if (length(colnames(sp))==6){  #This makes additional distinctions that can be used for species where Native/Introduced is designated
    sp.native<-sp[(sp$Native=='Native' & sp$Half.Error<=cutoff),]
    sp.intro<-sp[(sp$Native=='Introduced' & sp$Half.Error<=cutoff),]
  }
  # Create spatial objects out of the locations for all, native and introduced locations
  sp.pts<-SpatialPoints(sp[sp$Half.Error<=cutoff,2:3],proj4string=CRS(projection(environmental.data)))    # All points of occurrence
  sp.pts<-SpatialPointsDataFrame(sp.pts,data.frame(Internal.ID=sp[,1],sp[,4:length(colnames(sp))]),)                   # Attaches additional information
 
  # Thinning (if argument 'thin' is greater than zero then thin to a resolution no greater than argument)
  if(thin>0) sp.pts <- thin.points(sp.pts, resolution = thin, plot = FALSE)
  
  # Generate background points for presence-background estimators (i.e., MaxEnt and plug-and-play)
  set.seed(seed) #Random seed for repeatable background points (default) or variable for replicates is different values passed
  # Create a buffer around occurrence points from which to select background points
  if (buffer=="global") {
    backgr.pts<-randomPoints(environmental.data,backgr.n)# Large number of background points as in Phillips and Elith, taken from whole globe
    random.occur.pts<-randomPoints(environmental.data,length(sp.pts)) #Randomly distributed "occurrence" points for use in experiment
  }else{ #This is when background points are taken from buffer around occurrence rather than globally
    Spbuffpolys<-circles(sp.pts, d=buffer,lonlat=T)
    background<-gUnaryUnion(Spbuffpolys@polygons)
    backgraster<-mask(environmental.data,mask=background)                   # This masks the data to this region so that random points can be drawn
    backgr.pts<-randomPoints(backgraster,backgr.n)# Large number of background points as in Phillips and Elith
    random.occur.pts<-randomPoints(backgraster,length(sp.pts)) #Randomly distributed "occurrence" points for use in experiment
  }    
  # Check mapping
  if(check.map==TRUE){
    par(mfrow=c(2,1))
    plot(environmental.data[[1]], main=filename)
    if (buffer!="global") plot(background, add=TRUE)
    plot(environmental.data[[1]])
    plot(sp.pts[sp.pts@data$Native=='Native',],add=TRUE,col='darkgreen',cex=0.25)
    plot(sp.pts[sp.pts@data$Native=='Introduced',],add=TRUE,col='darkblue',cex=0.25)
    points(random.occur.pts,col="yellow",cex=0.2)
  }
  
  #Extract the environmental data at the occurrence points and background points
  covar.presence <- data.frame(extract(environmental.data, sp.pts))        # Extract environmental covariates at occurrence points
  covar.background<-data.frame(extract(environmental.data, backgr.pts))    # Extract environmental covariates at background points
  covar.random<-data.frame(extract(environmental.data,random.occur.pts))   # Extract environmental covariates at random occurrence points
  
  #Check for NA and remove
  z<-which(is.na(covar.presence))
  if(length(z)>0) covar.presence<-covar.presence[-z,] #This checks that there were some NA, then removes them, without NAs problems with z=numeric(0)
  covar.random<-covar.random[1:dim(covar.presence)[1],] #This reduces size of covar.random if NAs have been removed from covar.presence
  
  ##Divide into background and training data
  set.seed(482013) #Random seed for repeatable splits, any new splits need to be added after these to preserve the test data
  ##Same splits for different background and random points is fine because point locations are repeated or varied above (L34)
  train.presence.id <- sample(seq(1,dim(covar.presence)[1]),ceiling(train.fraction*dim(covar.presence[1])))  # Sample IDs for training data
  train.presence <- covar.presence[c(train.presence.id),]     # Environmental data for model fitting
  train.presence.pca <- predict(global.pca, train.presence)   # Environmental data for model fitting projected to PCA space
  test.presence<-covar.presence[-c(train.presence.id),]       # Environmental data for model testing
  test.presence.pca <- predict(global.pca, test.presence)     # Environmental data for model testing projected to PCA space
  
  train.background.id <- sample(seq(1,dim(covar.background)[1]),ceiling(train.fraction*dim(covar.background[1])))  # Sample IDs for training data
  train.background <- covar.background[c(train.background.id),]   # Environmental data for model fitting
  train.background.pca <- predict(global.pca, train.background)   # Environmental data for model fitting projected to PCA space
  test.background<-covar.background[-c(train.background.id),]     # Environmental data for model testing
  test.background.pca <- predict(global.pca, test.background)     # Environmental data for model testing projected to PCA space
  
  ##The following random points and validation point splits can be used for testing for overfitting without compromising the test data
  train.random.id <- sample(seq(1,dim(covar.random)[1]),ceiling(train.fraction*dim(covar.random[1])))  # Sample IDs for training data
  train.random <- covar.random[c(train.random.id),]   # Environmental data for model fitting
  train.random.pca <- predict(global.pca, train.random)   # Environmental data for model fitting projected to PCA space
  test.random<-covar.random[-c(train.random.id),]     # Environmental data for model testing
  test.random.pca <- predict(global.pca, test.random)     # Environmental data for model testing projected to PCA space
  
  train.validation.id <-sample(seq(1,length(train.presence.id)),ceiling((1-validation.fraction)*length(train.presence.id))) #Sample ids for the training data used in model validation
  train.validation <-train.presence[c(train.validation.id),] #Environmental data for model fitting during validation
  train.validation.pca <-train.presence.pca[c(train.validation.id),] #Environmental data for model fitting during validation projected to PCA space
  test.validation <-train.presence[-c(train.validation.id),] #Environmental data for testing the model during validation
  test.validation.pca <-train.presence.pca[-c(train.validation.id),] #Environmental data for testing the model during validation
  
  train.validation.random.id <-sample(seq(1,length(train.random.id)),ceiling((1-validation.fraction)*length(train.random.id))) #Sample ids for the training data used in model validation
  train.validation.random <-train.random[c(train.validation.random.id),] #Environmental data for model fitting during validation
  train.validation.random.pca <-train.random.pca[c(train.validation.random.id),] #Environmental data for model fitting during validation projected to PCA space
  test.validation.random <-train.random[-c(train.validation.random.id),] #Environmental data for testing the model during validation
  test.validation.random.pca <-train.random.pca[-c(train.validation.random.id),] #Environmental data for testing the model during validation
  
  n <- dim(train.presence)[1]   # Number of occurrence records in training dataset
  m <- dim(test.presence)[1]    # Number of occurrence records in testing dataset
  j <- dim(train.validation)[1] # Number of occurrence records in the subset of training used in training for validation, unneeded
  k <- dim(test.validation)[1] #Number of occurrence records in the subset of training used in testing for validation
  
  train.background.balanced <- train.background[1:n,]
  train.background.balanced.pca <- train.background.pca[1:n,] 
  train.validation.background <- train.background.balanced[1:j,]
  train.validation.background.pca <- train.background.balanced.pca[1:j,]
  test.validation.background <- train.background.balanced[1:k,]
  test.validation.background.pca <- train.background.balanced.pca[1:k,]
  
  test.background <- test.background[1:m,]
  test.background.pca <- test.background.pca[1:m,] 
  
  
  
  out<-(list(sp.pts, backgr.pts, train.presence, train.presence.pca, test.presence, test.presence.pca, 
             train.background, train.background.pca, test.background, test.background.pca, train.background.balanced, train.background.balanced.pca, 
             train.random, train.random.pca, test.random, test.random.pca,
             train.validation, train.validation.pca, test.validation, test.validation.pca,
             train.validation.background, train.validation.background.pca, test.validation.background,test.validation.background.pca,
             train.validation.random, train.validation.random.pca, test.validation.random, test.validation.random.pca))
  names(out)<-c("sp.pts", "backgr.pts", "train.presence", "train.presence.pca", "test.presence", "test.presence.pca",
                "train.background", "train.background.pca", "test.background", "test.background.pca", "train.background.balanced", "train.background.balanced.pca",
                "train.random", "train.random.pca", "test.random", "test.random.pca",
                "train.validation", "train.validation.pca", "test.validation", "test.validation.pca",
                "train.validation.background","train.validation.background.pca","test.validation.background","test.validation.background.pca",
                "train.validation.random", "train.validation.random.pca", "test.validation.random", "test.validation.random.pca")
  return(out)
}

